-- SkillSummary 1.4, by Wikwocket (wikwocket@gmail.com)
-- 
-- Displays your skills in a floating summary window.
-- Type /skillsummary or /ss for options.
-- 
-- Usage: /skillsummary [option] or /ss [option]. Options:
--   show|hide|toggle - show or hide summary window.
--   lock|unlock - toggle window position locking and click-through.
--   class|prof|sec|ride|weap|armor|lang - toggle displayed skills.
--   cap - toggle display of capped skills (290/290, etc).
--   color - toggle use of color to indicate skill level.
-- 
-- To do list:
--   Right-justification of skill numbers

-- Key bindings:
BINDING_HEADER_SKILLSUMMARY = "SkillSummary";
BINDING_NAME_SKILLSUMMARY_TOGGLE = "Show/Hide SkillSummary";

-- Constants:
local VERSION = "1.4";
local SKILLSUMMARY_HELP_1 = "Displays your skills in a floating summary window.";
local SKILLSUMMARY_HELP_2 = "Usage: /skillsummary or /ss [option]. Options:";
local SKILLSUMMARY_HELP_3 = "  show||hide||toggle - show or hide window.";
local SKILLSUMMARY_HELP_4 = "  lock||unlock - toggle window locking and click-through.";
local SKILLSUMMARY_HELP_5 = "  class||prof||sec||ride||weap||armor||lang - toggle skill display.";
local SKILLSUMMARY_HELP_6 = "  cap - toggle display of capped skills (290/290).";
local SKILLSUMMARY_HELP_7 = "  color - toggle use of color to indicate skill level.";

-- Print: helper function to send message to default chat frame
local function Print(text)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(text);
	end
end

-- OnLoad: register events and slash commands, print welcome message
function SkillSummaryOnLoad()
	SkillSummaryHoverFrame:RegisterEvent("VARIABLES_LOADED");
	SkillSummaryHoverFrame:RegisterEvent("SKILL_LINES_CHANGED");
	SkillSummaryHoverFrame:RegisterEvent("CHARACTER_POINTS_CHANGED");
	
	SlashCmdList["SKILLSUMMARY"] = SkillSummarySlashCommand;
	SLASH_SKILLSUMMARY1 = "/skillsummary";
	SLASH_SKILLSUMMARY2 = "/ss";
	
	Print("SkillSummary " .. VERSION .. " loaded; type /skillsummary or /ss for options.");
end

-- Event handler: respond to events
function SkillSummaryOnEvent()
	-- VARIABLES_LOADED: initialize
	if (event == "VARIABLES_LOADED") then
		SkillSummaryInitialize();
	-- SKILL_LINES_CHANGED or CHARACTER_POINTS_CHANGED: refresh and redraw
	else
		SkillSummaryRefresh();
	end
end

-- Initialize: do one-time initialization tasks
function SkillSummaryInitialize()
	-- load (or create) saved variables
	if (not SkillSummaryVariables) then
		SkillSummaryVariables = {};
		SkillSummaryVariables.showClassSkills = false;
		SkillSummaryVariables.showProfessionSkills = true;
		SkillSummaryVariables.showSecondarySkills = true;
		SkillSummaryVariables.showRidingSkills = false;
		SkillSummaryVariables.showWeaponSkills = true;
		SkillSummaryVariables.showArmorSkills = false;
		SkillSummaryVariables.showLanguageSkills = false;
		SkillSummaryVariables.showCappedSkills = true;
		SkillSummaryVariables.noColor = false;
		SkillSummaryVariables.show = true;
		SkillSummaryVariables.lock = false;
	end
	
	-- register with myAddOns
	if(myAddOnsFrame_Register) then
		local SkillSummaryMyAddOnsDetails = {
			name = "SkillSummary",
			description = "Displays your skills in a floating summary window",
			version = VERSION,
			author = "Wikwocket",
			email = "wikwocket@gmail.com",
			category = MYADDONS_CATEGORY_OTHERS,
			frame = "SkillSummaryHoverFrame",
		};
		local SkillSummaryMyAddOnsHelp = {};
		SkillSummaryMyAddOnsHelp[1] = "SkillSummary " .. VERSION ..
			"\n" .. SKILLSUMMARY_HELP_1 .. "\n" .. SKILLSUMMARY_HELP_2 ..
			"\n" .. SKILLSUMMARY_HELP_3 .. "\n" .. SKILLSUMMARY_HELP_4 ..
			"\n" .. SKILLSUMMARY_HELP_5 .. "\n" .. SKILLSUMMARY_HELP_6 ..
			"\n" .. SKILLSUMMARY_HELP_7;
		myAddOnsFrame_Register(SkillSummaryMyAddOnsDetails, SkillSummaryMyAddOnsHelp);
	end
	
	-- show or hide UI
	if (SkillSummaryVariables.show) then
		SkillSummaryShow();
	else
		SkillSummaryHide();
	end
	
	-- lock or unlock UI
	if (SkillSummaryVariables.lock) then
		SkillSummaryHoverFrame:EnableMouse(false);
	else
		SkillSummaryHoverFrame:EnableMouse(true);
	end
end

-- SlashCommand: handle slash commands
function SkillSummarySlashCommand(command)
	-- show frame
	if (command == "show" or command == "on") then
		SkillSummaryShow();
	-- hide frame
	elseif (command == "hide" or command == "off") then
		SkillSummaryHide();
	-- toggle frame visibility
	elseif (command == "toggle") then
		SkillSummaryToggle();
	-- toggle class skill visibility
	elseif (command == "class") then
		SkillSummaryVariables.showClassSkills = not SkillSummaryVariables.showClassSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of class skills toggled.");
	-- toggle profession skill visibility
	elseif (command == "prof" or command == "proffes") then
		SkillSummaryVariables.showProfessionSkills = not SkillSummaryVariables.showProfessionSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of profession skills toggled.");
	-- toggle secondary skill visibility
	elseif (command == "second" or command == "sec") then
		SkillSummaryVariables.showSecondarySkills = not SkillSummaryVariables.showSecondarySkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of secondary skills toggled.");
	-- toggle riding skill visibility
	elseif (command == "riding" or command == "ride") then
		SkillSummaryVariables.showRidingSkills = not SkillSummaryVariables.showRidingSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of riding skills toggled.");
	-- toggle weapon skill visibility
	elseif (command == "weapon" or command == "weap") then
		SkillSummaryVariables.showWeaponSkills = not SkillSummaryVariables.showWeaponSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of weapon skills toggled.");
	-- toggle weapon skill visibility
	elseif (command == "armor") then
		SkillSummaryVariables.showArmorSkills = not SkillSummaryVariables.showArmorSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of armor skills toggled.");
	-- toggle weapon skill visibility
	elseif (command == "lang" or command == "language") then
		SkillSummaryVariables.showLanguageSkills = not SkillSummaryVariables.showLanguageSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of language skills toggled.");
	-- toggle visibility of capped skills
	elseif (command == "cap" or command == "capped") then
		SkillSummaryVariables.showCappedSkills = not SkillSummaryVariables.showCappedSkills;
		SkillSummaryRefresh();
		Print("SkillSummary: Display of capped skills toggled.");
	-- toggle use of color
	elseif (command == "color" or command == "colors") then
		SkillSummaryVariables.noColor = not SkillSummaryVariables.noColor;
		SkillSummaryRefresh();
		Print("SkillSummary: Use of color toggled.");
	-- lock frame
	elseif (command == "lock") then
		SkillSummaryVariables.lock = true;
		SkillSummaryHoverFrame:EnableMouse(false);
		Print("SkillSummary: Window locked. You can now click through the window.");
	-- unlock frame
	elseif (command == "unlock") then
		SkillSummaryVariables.lock = false;
		SkillSummaryHoverFrame:EnableMouse(true);
		Print("SkillSummary: Window unlocked. You can now move the window by clicking and dragging.");
	-- default: print usage menu
	else
		Print("SkillSummary " .. VERSION);
		Print(SKILLSUMMARY_HELP_1);
		Print(SKILLSUMMARY_HELP_2);
		Print(SKILLSUMMARY_HELP_3);
		Print(SKILLSUMMARY_HELP_4);
		Print(SKILLSUMMARY_HELP_5);
		Print(SKILLSUMMARY_HELP_6);
		Print(SKILLSUMMARY_HELP_7);
	end
end

-- Show: show frame (after refreshing) and register update event
function SkillSummaryShow()
	SkillSummaryVariables.show = true;
	SkillSummaryRefresh();
	SkillSummaryHoverFrame:RegisterEvent("SKILL_LINES_CHANGED");
	SkillSummaryHoverFrame:RegisterEvent("CHARACTER_POINTS_CHANGED");
	SkillSummaryHoverFrame:Show();
	Print("SkillSummary: Window shown.");
end

-- Hide: hide frame and unregister update event
function SkillSummaryHide()
	SkillSummaryVariables.show = false;
	SkillSummaryHoverFrame:Hide();
	SkillSummaryHoverFrame:UnregisterEvent("SKILL_LINES_CHANGED");
	SkillSummaryHoverFrame:UnregisterEvent("CHARACTER_POINTS_CHANGED");
	Print("SkillSummary: Window hidden.");
end

-- Toggle: show or hide frame, depending on which state it's in
function SkillSummaryToggle()
	if (SkillSummaryHoverFrame:IsVisible()) then
		SkillSummaryHide();
	else
		SkillSummaryShow();
	end
end

-- Refresh: refresh the skills text, and set the frame's height and width
function SkillSummaryRefresh()
	local numSkills = GetNumSkillLines();
	local lastHeader = "";
	local skillSummaryText = "";
	local rankString = "";
	local percent = 0.0;
	-- go through all skills, picking which ones to add to summary
	for i = 1, numSkills do
		local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType = GetSkillLineInfo(i);
		-- remember which subsection we are in
		if (isHeader) then
			lastHeader = skillName;
		end
		-- choose which skills to add, based on saved preferences
		if ( not ( (not SkillSummaryVariables.showClassSkills and lastHeader == "Class Skills")
				or (not SkillSummaryVariables.showProfessionSkills and lastHeader == "Professions")
				or (not SkillSummaryVariables.showSecondarySkills and lastHeader == "Secondary Skills")
				or (not SkillSummaryVariables.showRidingSkills and lastHeader == "Secondary Skills" and skillMaxRank == 1)
				or (not SkillSummaryVariables.showWeaponSkills and lastHeader == "Weapon Skills")
				or (not SkillSummaryVariables.showArmorSkills and lastHeader == "Armor Proficiencies")
				or (not SkillSummaryVariables.showLanguageSkills and lastHeader == "Languages") ) ) then
					if (isHeader) then
						-- print headers in white
						skillSummaryText = skillSummaryText .. "|cffffffff" .. skillName .. ":|r\n";
					else
						-- replace "Two-handed" with "2H" for space
						if (lastHeader == "Weapon Skills" and string.sub(skillName, 0, 3) == "Two") then
							skillName = "2H " .. string.sub(skillName, 12);
						end
						-- remove redundant "Language: " for space
						if (string.sub(skillName, 0, 10) == "Language: ") then
							skillName = string.sub(skillName, 11);
						end
						-- add skill, if not capped or if printing capped skills
						if (SkillSummaryVariables.showCappedSkills or skillRank < skillMaxRank) then
							-- pick colors if desired, else white
							if (SkillSummaryVariables.noColor) then
								rankString = "|cffffffff" .. skillRank .. " / " .. skillMaxRank .. "|r";
							else
								percent = (skillRank - 1)/ (skillMaxRank - 1);
								if (percent >= 0.5) then
									rankString = "|cff" .. string.format("%02x", ((1.0 - percent) * 2 * 255)) .. "ff00" .. skillRank .. " / " .. skillMaxRank .. "|r";
								else
									rankString = "|cffff" .. string.format("%02x", (percent * 2 * 255)) .. "00" .. skillRank .. " / " .. skillMaxRank .. "|r";
								end
							end
							skillSummaryText = skillSummaryText .. "  " .. skillName .. ": " .. rankString .. "\n";
						end
					end
		end
	end
	if (skillSummaryText == "") then
		skillSummaryText = "No skills to show!"
	end
	
	-- set text of frame, and determine proper width and height
	local text1 = getglobal("SkillSummaryTitleText");
	local text2 = getglobal("SkillSummaryText");
	text2:SetText(skillSummaryText);
	local width1 = text1:GetWidth();
	local width2 = text2:GetWidth();
	if (width2 > width1) then
		width1 = width2;
	end
	SkillSummaryHoverFrame:SetWidth(width1 + 20);
	-- also set widths of buttons to that, if want to do anything on click?
	local height1 = text1:GetHeight();
	local height2 = text2:GetHeight();
	SkillSummaryHoverFrame:SetHeight(height1 + height2 + 30);
end
